/*********************************************
 * OPL 12.6.2.0 Model
 * Author: aguinet
 * Creation Date: 17 janv. 2018 at 16:47:52
 *********************************************/
 
 /* Paramtres */
int Neta = ...;
int Nper = ...;
float R1 = ...;
float R2 =...;
float R3 =...;
int Nvisit =...;
int Natt =...;
int Nlit =...;
int Npla =...;
int S =...;
int T =...;
int N1 =...;
int Cdec =...;
int Chosp =...;
int Chad =...;
int Ctrait=...;

/* Domaines de valeur */
range I = 1..Neta;
range P = 1..Nper;
range Q = T..Nper;
range K = 1..Nper-S;
range R = 2..Nper;
range H = 1..Nper-S+1;
range N = 2..N1;
range M = N1+1..Nper;

/* Variables de dcision */
dvar float+ EVL [I][P];
dvar float+ Res [P][I];
dvar float+ Vmax;
dvar float+ Iday;
dvar float+ Hday;
dvar float+ Coutinf;

/* Objectif : minimiser le cot des personnes dcdes et des personnes traites : Cdec euros d'indemnisation des proches pour un dcs, 
Chosp euros par priode en Soins Intensifs plus Chad euros en HAD par priode, Ctrait euros de soins infirmiers  domicile */
minimize sum(p in P) (EVL[8][p] * Cdec + EVL[6][p] * Chosp + EVL[7][p] * Chad + EVL[4][p] * Ctrait);

/* Avec les contraintes */
subject to
{
  
  /* Resistance de la population face au risque infectieux */
  EVL [1][1]==Nvisit;
  forall(p in N) EVL [1][p] == EVL [1][p-1] * (1-R1);
  forall(p in M) EVL [1][p] == 0;
  
  /* Progression des infections  taux constant */
  EVL [2][1] == 0;
  forall(p in R) EVL [2][p] == EVL [1][p-1] * R1;
  
  /* Apparition des symptomes (malades) et administation des traitements */
  EVL [3][1] == 0;  
  forall(p in R) EVL [3][p] == EVL [2][p-1] * R2 - EVL [4][p-1]; 
 
   /* Limitation des stocks d'antibiotiques, antibiotiques, antitoxines... en officines */
  EVL [4][1] == 0; 
  sum (p in P) EVL [4][p] <= Natt;

  /* Malades en cours d'agravation car non traits par faute de ressource */
  EVL [5][1] == 0;
  forall(p in R) EVL [5][p] == EVL [5][p-1] * (1-R3) + EVL [3][p-1] - EVL [6][p-1];
  
  /* Limitation du nombre de lits de soins intensifs pour un sjour de S priodes */
  EVL [6][1]== 0;
  forall(p in H) sum (q in p..p+(S-1)) EVL [6][q] <= Nlit;
  
  /* Malades admis en hospitalisation  domicile aprs soins intensifs cad soit aprs S priodes et pour T priodes */
  EVL [7][1] == 0; 
  EVL [7][2] == 0; 
  forall(p in K) EVL [7][p+S] == EVL [6][p];
 
  /* Limitation du nombre de places en HAD pour un sjour de T priodes */  
  forall(p in Q) sum (q in p-(T-1)..p) EVL [7][q] <= Npla;

  /* Nombre de malades dcdes */
  EVL [8][1] == 0;
  forall(p in R) EVL [8][p] == EVL [5][p-1] * R3;
  
  /* Indicateurs:  dcs (Vmax), lits de soins intensifs (Iday), places HAD utiliss (Hday), cot de la solution (Coutinf) */
  Vmax == sum (p in P) EVL [8][p];
  Iday == sum (p in P) EVL [6][p];
  Hday == sum (p in P) EVL [7][p];
  Coutinf == sum(p in P) (EVL[8][p] * Cdec + EVL[6][p] * Chosp + EVL[7][p] * Chad + EVL[4][p] * Ctrait);
  
  /* Dtail de l'infection sur l'horizon */
  forall (p in P, i in I) Res[p][i]==EVL[i][p];
}  
 